@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -39,7 +39,9 @@
  * (of class ListOfParameters), in addition to the attributes and
  * subelements it inherits from SBase.
  *
- * KineticLaw's "math" subelement for holding a MathML formula defines the
+ * KineticLaw's "math" subelement for holding a MathML formula (required 
+ * through SBML Level&nbsp;3 Version&nbsp;1, but optional as of SBML 
+ * Level&nbsp;3 Version&nbsp;2) defines the
  * rate of the reaction.  The formula may refer to other entities in a
  * model as well as local parameter definitions within the scope of the
  * Reaction (see below).  It is important to keep in mind, however, that
@@ -55,7 +57,9 @@
  * objects; in SBML Level&nbsp;3, this is achieved using a specialized
  * object class called LocalParameter and the containing subelement is
  * called "listOfLocalParameters".  In both cases, the parameters so
- * defined are only visible within the KineticLaw; they cannot be accessed
+ * defined are only visible within the KineticLaw (or, as of SBML
+ * Level&nbsp;3 Version&nbsp;2, only visible within the parent Reaction); 
+ * they cannot be accessed
  * outside.  A local parameter within one reaction is not visible from
  * within another reaction, nor is it visible to any other construct
  * outside of the KineticLaw in which it is defined.  In addition, another
@@ -105,6 +109,15 @@
  * As mentioned above, in SBML Level&nbsp;2 Versions 2&ndash;4, local
  * parameters are of class Parameter.  In SBML Level&nbsp;3, the class of
  * object is LocalParameter.
+ *
+ * In SBML Level&nbsp;3 Version&nbsp;2, the scope of the LocalParameter
+ * was expanded to the entire Reaction, instead of just the KineticLaw.  
+ * This introduced a single new restriction: an L3v2 LocalParameter may 
+ * not now shadow the @c id of any Species referenced by a SpeciesReference
+ * in the same Reaction.  Other than that, there is no difference in any 
+ * core construct.  However, packages may take advantage of this new scope by 
+ * adding elements to the Reaction that may now reference a LocalParameter 
+ * defined in the same Reaction.
  */
 
 /**
@@ -185,15 +198,12 @@ public:
    * Creates a new KineticLaw using the given SBML @p level and @p version
    * values.
    *
-   * @param level an unsigned int, the SBML Level to assign to this KineticLaw
+   * @param level an unsigned int, the SBML Level to assign to this KineticLaw.
    *
    * @param version an unsigned int, the SBML Version to assign to this
-   * KineticLaw
+   * KineticLaw.
    *
-   * @throws SBMLConstructorException
-   * Thrown if the given @p level and @p version combination, or this kind
-   * of SBML object, are either invalid or mismatched with respect to the
-   * parent SBMLDocument object.
+   * @copydetails doc_throw_exception_lv
    *
    * @copydetails doc_note_setting_lv
    */
@@ -208,10 +218,7 @@ public:
    *
    * @param sbmlns an SBMLNamespaces object.
    *
-   * @throws SBMLConstructorException
-   * Thrown if the given @p level and @p version combination, or this kind
-   * of SBML object, are either invalid or mismatched with respect to the
-   * parent SBMLDocument object.
+   * @copydetails doc_throw_exception_namespace
    *
    * @copydetails doc_note_setting_lv
    */
@@ -228,9 +235,6 @@ public:
    * Copy constructor; creates a copy of this KineticLaw.
    *
    * @param orig the object to copy.
-   * 
-   * @throws SBMLConstructorException
-   * Thrown if the argument @p orig is @c NULL.
    */
   KineticLaw (const KineticLaw& orig);
 
@@ -238,15 +242,13 @@ public:
   /**
    * Assignment operator for KineticLaw.
    *
-   * @param rhs The object whose values are used as the basis of the
+   * @param rhs the object whose values are used as the basis of the
    * assignment.
-   *
-   * @throws SBMLConstructorException
-   * Thrown if the argument @p rhs is @c NULL.
    */
   KineticLaw& operator=(const KineticLaw& rhs);
 
 
+  /** @cond doxygenLibsbmlInternal */
   /**
    * Accepts the given SBMLVisitor for this instance of KineticLaw.
    *
@@ -255,6 +257,7 @@ public:
    * @return the result of calling <code>v.visit()</code>.
    */
   virtual bool accept (SBMLVisitor& v) const;
+  /** @endcond */
 
 
   /**
@@ -269,7 +272,7 @@ public:
    * Returns the first child element found that has the given @p id in the
    * model-wide SId namespace, or @c NULL if no such object is found.
    *
-   * @param id string representing the id of objects to find.
+   * @param id string representing the id of the object to find.
    *
    * @return pointer to the first element found with the given @p id.
    */
@@ -280,7 +283,7 @@ public:
    * Returns the first child element it can find with the given @p metaid, or
    * @c NULL if no such object is found.
    *
-   * @param metaid string representing the metaid of objects to find
+   * @param metaid string representing the metaid of the object to find.
    *
    * @return pointer to the first element found with the given @p metaid.
    */
@@ -291,6 +294,10 @@ public:
    * Returns a List of all child SBase objects, including those nested to an
    * arbitrary depth
    *
+   * @param filter a pointer to an ElementFilter, which causes the function 
+   * to return only elements that match a particular set of constraints.  
+   * If NULL (the default), the function will return all child objects.
+   *
    * @return a List of pointers to all children objects.
    */
   virtual List* getAllElements(ElementFilter* filter=NULL);
@@ -324,11 +331,12 @@ public:
    * with SBML Level&nbsp;1, which represented mathematical formulas in
    * text-string form.
    * 
-   * @return the ASTNode representation of the mathematical formula.
+   * @return the ASTNode representation of the mathematical formula, 
+   * or NULL if the math is not set.
    *
    * @see getFormula()
    */
-  const ASTNode* getMath () const;
+  virtual const ASTNode* getMath () const;
 
 
   /**
@@ -369,7 +377,7 @@ public:
    * @note @htmlinclude level-1-uses-text-string-math.html
    *
    * @see isSetMath()
-   */  
+   */
   bool isSetFormula () const;
 
 
@@ -453,7 +461,7 @@ public:
    *
    * @see setFormula(const std::string& formula)
    */
-  int setMath (const ASTNode* math);
+  virtual int setMath (const ASTNode* math);
 
 
   /**
@@ -521,7 +529,7 @@ public:
    * Adds a copy of the given Parameter object to the list of local
    * parameters in this KineticLaw.
    *
-   * @param p the Parameter to add
+   * @param p the Parameter to add.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -542,7 +550,7 @@ public:
    * Adds a copy of the given LocalParameter object to the list of local
    * parameters in this KineticLaw.
    *
-   * @param p the LocalParameter to add
+   * @param p the LocalParameter to add.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -563,7 +571,7 @@ public:
    * Creates a new Parameter object, adds it to this KineticLaw's list of
    * local parameters, and returns the Parameter object created.
    *
-   * @return a new Parameter object instance
+   * @return a new Parameter object instance.
    *
    * @see addParameter(const Parameter* p)
    */
@@ -574,7 +582,7 @@ public:
    * Creates a new LocalParameter object, adds it to this KineticLaw's list
    * of local parameters, and returns the LocalParameter object created.
    *
-   * @return a new LocalParameter object instance
+   * @return a new LocalParameter object instance.
    *
    * @see addLocalParameter(const LocalParameter* p)
    */
@@ -617,7 +625,7 @@ public:
    * Returns the nth Parameter object in the list of local parameters in
    * this KineticLaw instance.
    *
-   * @param n the index of the Parameter object sought
+   * @param n the index of the Parameter object sought.
    * 
    * @return the nth Parameter of this KineticLaw.
    */
@@ -628,7 +636,7 @@ public:
    * Returns the nth Parameter object in the list of local parameters in
    * this KineticLaw instance.
    *
-   * @param n the index of the Parameter object sought
+   * @param n the index of the Parameter object sought.
    * 
    * @return the nth Parameter of this KineticLaw.
    */
@@ -639,7 +647,7 @@ public:
    * Returns the nth LocalParameter object in the list of local parameters in
    * this KineticLaw instance.
    *
-   * @param n the index of the LocalParameter object sought
+   * @param n the index of the LocalParameter object sought.
    * 
    * @return the nth LocalParameter of this KineticLaw.
    */
@@ -650,7 +658,7 @@ public:
    * Returns the nth LocalParameter object in the list of local parameters in
    * this KineticLaw instance.
    *
-   * @param n the index of the LocalParameter object sought
+   * @param n the index of the LocalParameter object sought.
    * 
    * @return the nth LocalParameter of this KineticLaw.
    */
@@ -795,7 +803,7 @@ public:
    *
    * The caller owns the returned object and is responsible for deleting it.
    *
-   * @param n the index of the Parameter object to remove
+   * @param n the index of the Parameter object to remove.
    * 
    * @return the Parameter object removed.  As mentioned above, 
    * the caller owns the returned item. @c NULL is returned if the given index 
@@ -810,7 +818,7 @@ public:
    *
    * The caller owns the returned object and is responsible for deleting it.
    *
-   * @param n the index of the LocalParameter object to remove
+   * @param n the index of the LocalParameter object to remove.
    * 
    * @return the LocalParameter object removed.  As mentioned above, 
    * the caller owns the returned item. @c NULL is returned if the given index 
@@ -825,7 +833,7 @@ public:
    *
    * The caller owns the returned object and is responsible for deleting it.
    *
-   * @param sid the identifier of the Parameter to remove
+   * @param sid the identifier of the Parameter to remove.
    * 
    * @return the Parameter object removed.  As mentioned above, the 
    * caller owns the returned object. @c NULL is returned if no Parameter
@@ -840,7 +848,7 @@ public:
    *
    * The caller owns the returned object and is responsible for deleting it.
    *
-   * @param sid the identifier of the LocalParameter to remove
+   * @param sid the identifier of the LocalParameter to remove.
    * 
    * @return the LocalParameter object removed.  As mentioned above, the 
    * caller owns the returned object. @c NULL is returned if no LocalParameter
@@ -923,7 +931,7 @@ public:
   /** @cond doxygenLibsbmlInternal */
   /**
    * Subclasses should override this method to write out their contained
-   * SBML objects as XML elements.  Be sure to call your parents
+   * SBML objects as XML elements.  Be sure to call your parent's
    * implementation of this method as well.
    */
   virtual void writeElements (XMLOutputStream& stream) const;
@@ -948,7 +956,8 @@ public:
    * KineticLaw object have been set.
    *
    * @note The required elements for a KineticLaw object are:
-   * @li "math"
+   * @li "math" inSBML Level&nbsp;2 and Level&nbsp;3 Version&nbsp;1.  
+   *     (In SBML Level&nbsp;3 Version&nbsp;2+, it is no longer required.)
    *
    * @return a boolean value indicating whether all the required
    * elements for this object have been defined.
@@ -1013,9 +1022,336 @@ public:
 
 
 
-protected:
+
+
+
+  #ifndef SWIG
+
+
+
   /** @cond doxygenLibsbmlInternal */
 
+  /**
+   * Gets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName, bool& value)
+    const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName, int& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           double& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           unsigned int& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           std::string& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           const char* value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Predicate returning @c true if this KineticLaw's attribute "attributeName"
+   * is set.
+   *
+   * @param attributeName, the name of the attribute to query.
+   *
+   * @return @c true if this KineticLaw's attribute "attributeName" has been
+   * set, otherwise @c false is returned.
+   */
+  virtual bool isSetAttribute(const std::string& attributeName) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, bool value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, int value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, double value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName,
+                           unsigned int value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName,
+                           const std::string& value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, const char*
+    value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Unsets the value of the "attributeName" attribute of this KineticLaw.
+   *
+   * @param attributeName, the name of the attribute to query.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int unsetAttribute(const std::string& attributeName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Creates and returns an new "elementName" object in this KineticLaw.
+   *
+   * @param elementName, the name of the element to create.
+   *
+   * pointer to the element created.
+   */
+  virtual SBase* createChildObject(const std::string& elementName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Returns the number of "elementName" in this KineticLaw.
+   *
+   * @param elementName, the name of the element to get number of.
+   *
+   * unsigned int number of elements.
+   */
+  virtual unsigned int getNumObjects(const std::string& elementName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Returns the nth object of "objectName" in this KineticLaw.
+   *
+   * @param elementName, the name of the element to get number of.
+   *
+   * @param index, unsigned int teh index of teh object to retrieve.
+   *
+   * pointer to the object.
+   */
+  virtual SBase* getObject(const std::string& elementName, unsigned int index);
+
+  /** @endcond */
+
+
+
+
+  #endif /* !SWIG */
+
+
+protected:
+  /** @cond doxygenLibsbmlInternal */
   /**
    * Create and return an SBML object of this class, if present.
    *
@@ -1029,7 +1365,7 @@ protected:
    * Subclasses should override this method to read (and store) XHTML,
    * MathML, etc. directly from the XMLInputStream.
    *
-   * @return true if the subclass read from the stream, false otherwise.
+   * @return @c true if the subclass read from the stream, @c false otherwise.
    */
   virtual bool readOtherXML (XMLInputStream& stream);
 
@@ -1046,7 +1382,7 @@ protected:
   /**
    * Subclasses should override this method to read values from the given
    * XMLAttributes set into their specific fields.  Be sure to call your
-   * parents implementation of this method as well.
+   * parent's implementation of this method as well.
    */
   virtual void readAttributes (const XMLAttributes& attributes,
                                const ExpectedAttributes& expectedAttributes);
@@ -1061,7 +1397,7 @@ protected:
 
   /**
    * Subclasses should override this method to write their XML attributes
-   * to the XMLOutputStream.  Be sure to call your parents implementation
+   * to the XMLOutputStream.  Be sure to call your parent's implementation
    * of this method as well.
    */
   virtual void writeAttributes (XMLOutputStream& stream) const;
@@ -1115,10 +1451,10 @@ BEGIN_C_DECLS
  * and @p version values.
  *
  * @param level an unsigned int, the SBML Level to assign to this
- * KineticLaw_t
+ * KineticLaw_t.
  *
  * @param version an unsigned int, the SBML Version to assign to this
- * KineticLaw_t
+ * KineticLaw_t.
  *
  * @return a pointer to the newly created KineticLaw_t structure.
  *
@@ -1142,7 +1478,7 @@ KineticLaw_create (unsigned int level, unsigned int version);
  * SBMLNamespaces_t structure.
  *
  * @param sbmlns SBMLNamespaces_t, a pointer to an SBMLNamespaces_t structure
- * to assign to this KineticLaw_t
+ * to assign to this KineticLaw_t.
  *
  * @return a pointer to the newly created KineticLaw_t structure.
  *
@@ -1164,7 +1500,7 @@ KineticLaw_createWithNS (SBMLNamespaces_t *sbmlns);
 /**
  * Frees the given KineticLaw_t structure.
  *
- * @param kl the KineticLaw_t structure
+ * @param kl the KineticLaw_t structure.
  *
  * @memberof KineticLaw_t
  */
@@ -1191,7 +1527,7 @@ KineticLaw_clone (const KineticLaw_t *kl);
  * Returns a list of XMLNamespaces_t associated with this KineticLaw_t
  * structure.
  *
- * @param kl the KineticLaw_t structure
+ * @param kl the KineticLaw_t structure.
  * 
  * @return pointer to the XMLNamespaces_t structure associated with 
  * this structure
@@ -1429,7 +1765,7 @@ KineticLaw_setFormula (KineticLaw_t *kl, const char *formula);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param math an ASTNode_t structure representing the mathematical formula
+ * @param math an ASTNode_t structure representing the mathematical formula.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1449,7 +1785,7 @@ KineticLaw_setMath (KineticLaw_t *kl, const ASTNode_t *math);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param sid the identifier of the units
+ * @param sid the identifier of the units.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1477,7 +1813,7 @@ KineticLaw_setTimeUnits (KineticLaw_t *kl, const char *sid);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param sid the identifier of the units
+ * @param sid the identifier of the units.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1544,7 +1880,7 @@ KineticLaw_unsetSubstanceUnits (KineticLaw_t *kl);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param p a pointer to a Parameter_t structure
+ * @param p a pointer to a Parameter_t structure.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1566,7 +1902,7 @@ KineticLaw_addParameter (KineticLaw_t *kl, const Parameter_t *p);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param p a pointer to a LocalParameter_t structure
+ * @param p a pointer to a LocalParameter_t structure.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1653,7 +1989,7 @@ KineticLaw_getListOfLocalParameters (KineticLaw_t *kl);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param n the index of the Parameter_t structure sought
+ * @param n the index of the Parameter_t structure sought.
  * 
  * @return a pointer to the Parameter_t structure
  *
@@ -1675,7 +2011,7 @@ KineticLaw_getParameter (KineticLaw_t *kl, unsigned int n);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param n the index of the LocalParameter_t structure sought
+ * @param n the index of the LocalParameter_t structure sought.
  * 
  * @return a pointer to the LocalParameter_t structure
  *
@@ -1694,7 +2030,7 @@ KineticLaw_getLocalParameter (KineticLaw_t *kl, unsigned int n);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param sid the identifier of the Parameter_t structure sought
+ * @param sid the identifier of the Parameter_t structure sought.
  * 
  * @return the Parameter_t structure with the given @p id, or @c NULL if no such
  * Parameter_t exists in the given KineticLaw_t structure.
@@ -1712,7 +2048,7 @@ KineticLaw_getParameterById (KineticLaw_t *kl, const char *sid);
  *
  * @param kl the KineticLaw_t structure.
  *
- * @param sid the identifier of the LocalParameter_t structure sought
+ * @param sid the identifier of the LocalParameter_t structure sought.
  * 
  * @return the LocalParameter_t structure with the given @p id, or @c NULL if no such
  * LocalParameter_t exists in the given KineticLaw_t structure.
@@ -1811,8 +2147,8 @@ KineticLaw_containsUndeclaredUnits(KineticLaw_t *kl);
  *
  * The caller owns the returned structure and is responsible for deleting it.
  *
- * @param kl the KineticLaw_t structure
- * @param n the integer index of the Parameter_t sought
+ * @param kl the KineticLaw_t structure.
+ * @param n the integer index of the Parameter_t sought.
  *
  * @return the Parameter_t structure removed.  As mentioned above, 
  * the caller owns the returned item. @c NULL is returned if the given index 
@@ -1831,8 +2167,8 @@ KineticLaw_removeParameter (KineticLaw_t *kl, unsigned int n);
  *
  * The caller owns the returned structure and is responsible for deleting it.
  *
- * @param kl the KineticLaw_t structure
- * @param n the integer index of the LocalParameter_t sought
+ * @param kl the KineticLaw_t structure.
+ * @param n the integer index of the LocalParameter_t sought.
  *
  * @return the LocalParameter_t structure removed.  As mentioned above, 
  * the caller owns the returned item. @c NULL is returned if the given index 
@@ -1852,8 +2188,8 @@ KineticLaw_removeLocalParameter (KineticLaw_t *kl, unsigned int n);
  *
  * The caller owns the returned structure and is responsible for deleting it.
  *
- * @param kl the KineticLaw_t structure
- * @param sid the string of the "id" attribute of the Parameter_t sought
+ * @param kl the KineticLaw_t structure.
+ * @param sid the string of the "id" attribute of the Parameter_t sought.
  *
  * @return the Parameter_t structure removed.  As mentioned above, the 
  * caller owns the returned structure. @c NULL is returned if no KineticLaw_t
@@ -1873,8 +2209,8 @@ KineticLaw_removeParameterById (KineticLaw_t *kl, const char *sid);
  *
  * The caller owns the returned structure and is responsible for deleting it.
  *
- * @param kl the KineticLaw_t structure
- * @param sid the string of the "id" attribute of the LocalParameter_t sought
+ * @param kl the KineticLaw_t structure.
+ * @param sid the string of the "id" attribute of the LocalParameter_t sought.
  *
  * @return the LocalParameter_t structure removed.  As mentioned above, the 
  * caller owns the returned structure. @c NULL is returned if no KineticLaw_t
