@@ -8,7 +8,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -53,9 +53,7 @@
 #include <sbml/extension/SBasePlugin.h>
 
 /** @cond doxygenIgnored */
-
 using namespace std;
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -122,29 +120,21 @@ SpeciesReference::~SpeciesReference ()
 /*
  * Copy constructor. Creates a copy of this SpeciesReference.
  */
-SpeciesReference::SpeciesReference (const SpeciesReference& orig) :
-   SimpleSpeciesReference( orig                )
- , mStoichiometryMath    ( NULL                   )
+SpeciesReference::SpeciesReference (const SpeciesReference& orig)
+ : SimpleSpeciesReference( orig                )
+ , mStoichiometry        ( orig.mStoichiometry )
+ , mDenominator          ( orig.mDenominator   )
+ , mStoichiometryMath    ( NULL                )
+ , mConstant             ( orig.mConstant)
+ , mIsSetConstant        ( orig.mIsSetConstant)
+ , mIsSetStoichiometry   ( orig.mIsSetStoichiometry)
+ , mExplicitlySetStoichiometry ( orig.mExplicitlySetStoichiometry)
+ , mExplicitlySetDenominator   ( orig.mExplicitlySetDenominator)
 {
-  if (&orig == NULL)
+  if (orig.mStoichiometryMath != NULL)
   {
-    throw SBMLConstructorException("Null argument to copy constructor");
-  }
-  else
-  {
-    mStoichiometry = orig.mStoichiometry ;
-    mDenominator = orig.mDenominator   ;
-    mConstant = orig.mConstant;
-    mIsSetConstant = orig.mIsSetConstant;
-    mIsSetStoichiometry = orig.mIsSetStoichiometry;
-    mExplicitlySetStoichiometry = orig.mExplicitlySetStoichiometry;
-    mExplicitlySetDenominator = orig.mExplicitlySetDenominator;
-
-    if (orig.mStoichiometryMath != NULL)
-    {
-      mStoichiometryMath = new StoichiometryMath(*orig.getStoichiometryMath());
+    mStoichiometryMath = new StoichiometryMath(*orig.getStoichiometryMath());
     mStoichiometryMath->connectToParent(this);
-    }
   }
 }
 
@@ -154,11 +144,7 @@ SpeciesReference::SpeciesReference (const SpeciesReference& orig) :
  */
 SpeciesReference& SpeciesReference::operator=(const SpeciesReference& rhs)
 {
-  if (&rhs == NULL)
-  {
-    throw SBMLConstructorException("Null argument to assignment operator");
-  }
-  else if(&rhs!=this)
+  if(&rhs!=this)
   {
     this->SBase::operator =(rhs);
     this->SimpleSpeciesReference::operator = ( rhs );
@@ -186,13 +172,7 @@ SpeciesReference& SpeciesReference::operator=(const SpeciesReference& rhs)
 }
 
 
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Reaction's next
- * SpeciesReference (if available).
- */
+/** @cond doxygenLibsbmlInternal */
 bool
 SpeciesReference::accept (SBMLVisitor& v) const
 {
@@ -202,6 +182,7 @@ SpeciesReference::accept (SBMLVisitor& v) const
   
   return result;
 }
+/** @endcond */
 
 
 /*
@@ -448,7 +429,7 @@ SpeciesReference::unsetStoichiometryMath ()
 int
 SpeciesReference::unsetStoichiometry ()
 {
-  const int level = getLevel();
+  const unsigned int level = getLevel();
 
   if ( level > 2 )
   {
@@ -583,6 +564,420 @@ SpeciesReference::hasRequiredAttributes() const
 }
 
 
+
+
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::getAttribute(const std::string& attributeName,
+                               bool& value) const
+{
+  int return_value = SimpleSpeciesReference::getAttribute(attributeName,
+    value);
+
+  if (return_value == LIBSBML_OPERATION_SUCCESS)
+  {
+    return return_value;
+  }
+
+  if (attributeName == "constant")
+  {
+    value = getConstant();
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::getAttribute(const std::string& attributeName,
+                               int& value) const
+{
+  int return_value = SimpleSpeciesReference::getAttribute(attributeName,
+    value);
+
+  if (attributeName == "stoichiometry")
+  {
+    value = (int)(getStoichiometry());
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+  else if (attributeName == "denominator")
+  {
+    value = getDenominator();
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::getAttribute(const std::string& attributeName,
+                               double& value) const
+{
+  int return_value = SimpleSpeciesReference::getAttribute(attributeName,
+    value);
+
+  if (return_value == LIBSBML_OPERATION_SUCCESS)
+  {
+    return return_value;
+  }
+
+  if (attributeName == "stoichiometry")
+  {
+    value = getStoichiometry();
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::getAttribute(const std::string& attributeName,
+                               unsigned int& value) const
+{
+  int return_value = SimpleSpeciesReference::getAttribute(attributeName,
+    value);
+
+  if (return_value == LIBSBML_OPERATION_SUCCESS)
+  {
+    return return_value;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::getAttribute(const std::string& attributeName,
+                               std::string& value) const
+{
+  int return_value = SimpleSpeciesReference::getAttribute(attributeName,
+    value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::getAttribute(const std::string& attributeName,
+                               const char* value) const
+{
+  int return_value = SimpleSpeciesReference::getAttribute(attributeName,
+    value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Predicate returning @c true if this SpeciesReference's attribute
+ * "attributeName" is set.
+ */
+bool
+SpeciesReference::isSetAttribute(const std::string& attributeName) const
+{
+  bool value = SimpleSpeciesReference::isSetAttribute(attributeName);
+
+  if (attributeName == "stoichiometry")
+  {
+    value = isSetStoichiometry();
+  }
+  else if (attributeName == "constant")
+  {
+    value = isSetConstant();
+  }
+  else if (attributeName == "denominator")
+  {
+    value = true;
+  }
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::setAttribute(const std::string& attributeName, bool value)
+{
+  int return_value = SimpleSpeciesReference::setAttribute(attributeName,
+    value);
+
+  if (attributeName == "constant")
+  {
+    return_value = setConstant(value);
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::setAttribute(const std::string& attributeName, int value)
+{
+  int return_value = SimpleSpeciesReference::setAttribute(attributeName,
+    value);
+
+  if (attributeName == "stoichiometry")
+  {
+    return_value = setStoichiometry(value);
+  }
+  else if (attributeName == "denominator")
+  {
+    return_value = setDenominator(value);
+  }
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::setAttribute(const std::string& attributeName, double value)
+{
+  int return_value = SimpleSpeciesReference::setAttribute(attributeName,
+    value);
+
+  if (attributeName == "stoichiometry")
+  {
+    return_value = setStoichiometry(value);
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::setAttribute(const std::string& attributeName,
+                               unsigned int value)
+{
+  int return_value = SimpleSpeciesReference::setAttribute(attributeName,
+    value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::setAttribute(const std::string& attributeName,
+                               const std::string& value)
+{
+  int return_value = SimpleSpeciesReference::setAttribute(attributeName,
+    value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::setAttribute(const std::string& attributeName,
+                               const char* value)
+{
+  int return_value = SimpleSpeciesReference::setAttribute(attributeName,
+    value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Unsets the value of the "attributeName" attribute of this SpeciesReference.
+ */
+int
+SpeciesReference::unsetAttribute(const std::string& attributeName)
+{
+  int value = SimpleSpeciesReference::unsetAttribute(attributeName);
+
+  if (attributeName == "stoichiometry")
+  {
+    value = unsetStoichiometry();
+  }
+  else if (attributeName == "constant")
+  {
+    value = unsetConstant();
+  }
+  else if (attributeName == "denominator")
+  {
+    mDenominator = 1;
+    value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return value;
+}
+
+/** @endcond */
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Creates and returns an new "elementName" object in this SpeciesReference.
+ */
+SBase*
+SpeciesReference::createChildObject(const std::string& elementName)
+{
+  SBase* obj = NULL;
+
+  if (elementName == "stoichiometryMath")
+  {
+    return createStoichiometryMath();
+  }
+
+  return obj;
+}
+
+/** @endcond */
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Returns the number of "elementName" in this SpeciesReference.
+ */
+unsigned int
+SpeciesReference::getNumObjects(const std::string& elementName)
+{
+  unsigned int n = 0;
+
+  if (elementName == "stoichiometryMath")
+  {
+    if (isSetStoichiometryMath())
+    {
+      return 1;
+    }
+  }
+
+  return n;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Returns the nth object of "objectName" in this SpeciesReference.
+ */
+SBase*
+SpeciesReference::getObject(const std::string& elementName, unsigned int index)
+{
+  SBase* obj = NULL;
+
+  if (elementName == "stoichiometryMath")
+  {
+    return getStoichiometryMath();
+  }
+
+  return obj;
+}
+
+/** @endcond */
+
+
+
+
 /*
  * Sets the annotation of this SBML object to a copy of annotation.
  */
@@ -941,7 +1336,7 @@ SpeciesReference::addExpectedAttributes(ExpectedAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 SpeciesReference::readAttributes (const XMLAttributes& attributes,
@@ -971,7 +1366,7 @@ SpeciesReference::readAttributes (const XMLAttributes& attributes,
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 SpeciesReference::readL1Attributes (const XMLAttributes& attributes)
@@ -1007,7 +1402,7 @@ SpeciesReference::readL1Attributes (const XMLAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 SpeciesReference::readL2Attributes (const XMLAttributes& attributes)
@@ -1024,7 +1419,7 @@ SpeciesReference::readL2Attributes (const XMLAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 SpeciesReference::readL3Attributes (const XMLAttributes& attributes)
@@ -1063,7 +1458,7 @@ SpeciesReference::readL3Attributes (const XMLAttributes& attributes)
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write their XML attributes
- * to the XMLOutputStream.  Be sure to call your parents implementation
+ * to the XMLOutputStream.  Be sure to call your parent's implementation
  * of this method as well.
  */
 void
@@ -1116,7 +1511,7 @@ SpeciesReference::writeAttributes (XMLOutputStream& stream) const
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write out their contained
- * SBML objects as XML elements.  Be sure to call your parents
+ * SBML objects as XML elements.  Be sure to call your parent's
  * implementation of this method as well.
  */
 void
@@ -1244,12 +1639,12 @@ ListOfSpeciesReferences::getElementName () const
  */
 struct IdEqSSR : public unary_function<SBase*, bool>
 {
-  const string& id;
+  const string& mId;
 
-  IdEqSSR (const string& id) : id(id) { }
+  IdEqSSR (const string& id) : mId(id) { }
   bool operator() (SBase* sb)
-       { return (static_cast <SimpleSpeciesReference *> (sb)->getId()  == id) 
-         || (static_cast <SimpleSpeciesReference *> (sb)->getSpecies() == id); } 
+       { return (static_cast <SimpleSpeciesReference *> (sb)->getId()  == mId)
+         || (static_cast <SimpleSpeciesReference *> (sb)->getSpecies() == mId); } 
 };
 
 
@@ -1284,16 +1679,9 @@ ListOfSpeciesReferences::get (const std::string& sid) const
 {
   vector<SBase*>::const_iterator result;
 
-  if (&(sid) == NULL)
-  {
-    return NULL;
-  }
-  else
-  {
-    result = find_if( mItems.begin(), mItems.end(), IdEqSSR(sid) );
-    return (result == mItems.end()) ? NULL : 
-                             static_cast <SimpleSpeciesReference*> (*result);
-  }
+  result = find_if( mItems.begin(), mItems.end(), IdEqSSR(sid) );
+  return (result == mItems.end()) ? NULL : 
+                           static_cast <SimpleSpeciesReference*> (*result);
 }
 
 
@@ -1312,15 +1700,12 @@ ListOfSpeciesReferences::remove (const std::string& sid)
   SBase* item = NULL;
   vector<SBase*>::iterator result;
 
-  if (&(sid) != NULL)
-  {
-    result = find_if( mItems.begin(), mItems.end(), IdEqSSR(sid) );
+  result = find_if( mItems.begin(), mItems.end(), IdEqSSR(sid) );
 
-    if (result != mItems.end())
-    {
-      item = *result;
-      mItems.erase(result);
-    }
+  if (result != mItems.end())
+  {
+    item = *result;
+    mItems.erase(result);
   }
 
   return static_cast <SimpleSpeciesReference*> (item);
@@ -1471,7 +1856,6 @@ ListOfSpeciesReferences::createObject (XMLInputStream& stream)
 
 #endif /* __cplusplus */
 /** @cond doxygenIgnored */
-
 LIBSBML_EXTERN
 SpeciesReference_t *
 SpeciesReference_create (unsigned int level, unsigned int version)
@@ -1924,8 +2308,6 @@ ListOfSpeciesReferences_removeById (ListOf_t *lo, const char *sid)
   else
     return NULL;
 }
-
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_END
