@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -48,10 +48,10 @@
 
 #include <sbml/util/ElementFilter.h>
 
+#include <math.h>
+
 /** @cond doxygenIgnored */
-
 using namespace std;
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -59,8 +59,6 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 
 UnitDefinition::UnitDefinition (unsigned int level, unsigned int version) :
    SBase  ( level, version )
- , mId    ( ""       )
- , mName  ( ""       )
  , mUnits (level, version)
 {
   if (!hasValidLevelVersionNamespaceCombination())
@@ -72,8 +70,6 @@ UnitDefinition::UnitDefinition (unsigned int level, unsigned int version) :
 
 UnitDefinition::UnitDefinition (SBMLNamespaces * sbmlns) :
    SBase  ( sbmlns )
- , mId    ( ""       )
- , mName  ( ""       )
  , mUnits (sbmlns)
 {
   if (!hasValidLevelVersionNamespaceCombination())
@@ -98,19 +94,9 @@ UnitDefinition::~UnitDefinition ()
  * Copy constructor. Creates a copy of this UnitDefinition.
  */
 UnitDefinition::UnitDefinition(const UnitDefinition& orig) :
-    SBase     ( orig                    )
-  , mUnits    ( orig.mUnits             )
+    SBase     ( orig )
+  , mUnits    ( orig.mUnits )
 {
-  if (&orig == NULL)
-  {
-    throw SBMLConstructorException("Null argument to copy constructor");
-  }
-  else
-  {
-    mId    = orig.mId;
-    mName  = orig.mName;
-  }
-
   connectToChild();
 }
 
@@ -120,15 +106,9 @@ UnitDefinition::UnitDefinition(const UnitDefinition& orig) :
  */
 UnitDefinition& UnitDefinition::operator=(const UnitDefinition& rhs)
 {
-  if (&rhs == NULL)
-  {
-    throw SBMLConstructorException("Null argument to assignment operator");
-  }
-  else if(&rhs!=this)
+  if(&rhs!=this)
   {
     this->SBase::operator =(rhs);
-    mId = rhs.mId;
-    mName = rhs.mName;
     mUnits = rhs.mUnits;
   }
 
@@ -138,13 +118,7 @@ UnitDefinition& UnitDefinition::operator=(const UnitDefinition& rhs)
 }
 
 
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Model's next
- * UnitDefinition (if available).
- */
+/** @cond doxygenLibsbmlInternal */
 bool
 UnitDefinition::accept (SBMLVisitor& v) const
 {
@@ -153,7 +127,8 @@ UnitDefinition::accept (SBMLVisitor& v) const
 
   return result;
 }
-
+/** @endcond */
+\
 
 /*
  * @return a (deep) copy of this UnitDefinition.
@@ -259,11 +234,7 @@ UnitDefinition::setId (const std::string& sid)
     return LIBSBML_UNEXPECTED_ATTRIBUTE;
   }
 */
-  if (&(sid) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (!(SyntaxChecker::isValidInternalSId(sid)))
+  if (!(SyntaxChecker::isValidInternalSId(sid)))
   {
     return LIBSBML_INVALID_ATTRIBUTE_VALUE;
   }
@@ -284,11 +255,7 @@ UnitDefinition::setName (const std::string& name)
   /* if this is setting an L2 name the type is string
    * whereas if it is setting an L1 name its type is SId
    */
-  if (&(name) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (getLevel() == 1)
+  if (getLevel() == 1)
   {
     if (!(SyntaxChecker::isValidInternalSId(name)))
     {
@@ -344,17 +311,30 @@ UnitDefinition::unsetName ()
  * or multiplier values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfArea () const
+UnitDefinition::isVariantOfArea (bool relaxed) const
 {
   bool result = false;
 
   UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
   UnitDefinition::simplify(ud);
 
-  if (ud->getNumUnits() == 1)
+  if (!relaxed) 
   {
-    const Unit* u = ud->getUnit(0);
-    result        = u->isMetre() && u->getExponent() == 2;
+    // should be metre^2
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isMetre() && u->getExponent() == 2;
+    }
+  }
+  else
+  {
+    // should be metre
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isMetre();
+    }
   }
 
   delete ud;
@@ -368,17 +348,30 @@ UnitDefinition::isVariantOfArea () const
  * or multiplier values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfLength () const
+UnitDefinition::isVariantOfLength (bool relaxed) const
 {
   bool result = false;
 
   UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
   UnitDefinition::simplify(ud);
 
-  if (ud->getNumUnits() == 1)
+  if (!relaxed)
   {
-    const Unit* u = ud->getUnit(0);
-    result        = u->isMetre() && u->getExponent() == 1;
+    //should be metre^1
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isMetre() && u->getExponent() == 1;
+    }
+  }
+  else
+  {
+    //should be metre
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isMetre();
+    }
   }
 
   delete ud;
@@ -392,7 +385,7 @@ UnitDefinition::isVariantOfLength () const
  * scale or multiplier values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfSubstance () const
+UnitDefinition::isVariantOfSubstance (bool relaxed) const
 {
   bool result = false;
 
@@ -402,26 +395,47 @@ UnitDefinition::isVariantOfSubstance () const
   UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
   UnitDefinition::simplify(ud);
 
-  if (ud->getNumUnits() == 1)
+  if (!relaxed)
   {
-    const Unit* u = ud->getUnit(0);
-    if (level == 2 && version > 1)
+    // should be L1/L2V1: mole^1/item^1
+    //                L2: mole/item/gram/kilogram ^1
+    //                L3: mole/item/gram/kilogram/avogardo ^1
+    if (ud->getNumUnits() == 1)
     {
-      result = ((  u->isMole() || u->isItem() 
-                || u->isGram() || u->isKilogram())
-                && u->getExponent() == 1);
+      const Unit* u = ud->getUnit(0);
+      if (level == 2 && version > 1)
+      {
+        result = ((  u->isMole() || u->isItem() 
+                  || u->isGram() || u->isKilogram())
+                  && u->getExponent() == 1);
+      }
+      else if (level > 2)
+      {
+        result = ((  u->isMole() || u->isItem() 
+                  || u->isGram() || u->isKilogram()
+                  || u->isAvogadro())
+                  && u->getExponent() == 1);
+      }
+      else
+      {
+        result        = (u->isMole() || u->isItem()) 
+                      && u->getExponent() == 1;
+      }
     }
-    else if (level > 2)
+  }
+  else
+  {
+    // should be any combination of mole/item/gram/kilogram/avogardo ^1
+    unsigned int i = 0;
+    result = true;
+    while (result && i < ud->getNumUnits())
     {
+      const Unit* u = ud->getUnit(i);
       result = ((  u->isMole() || u->isItem() 
                 || u->isGram() || u->isKilogram()
-                || u->isAvogadro())
-                && u->getExponent() == 1);
-    }
-    else
-    {
-      result        = (u->isMole() || u->isItem()) 
-                    && u->getExponent() == 1;
+                || u->isAvogadro()));
+      
+      i++;
     }
   }
 
@@ -436,17 +450,30 @@ UnitDefinition::isVariantOfSubstance () const
  * or multiplier values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfTime () const
+UnitDefinition::isVariantOfTime (bool relaxed) const
 {
   bool result = false;
 
   UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
   UnitDefinition::simplify(ud);
 
-  if (ud->getNumUnits() == 1)
+  if (!relaxed)
   {
-    const Unit* u = ud->getUnit(0);
-    result        = u->isSecond() && u->getExponent() == 1;
+    // unit should simplify to second with exponent 1
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isSecond() && u->getExponent() == 1;
+    }
+  }
+  else
+  {
+    // from l3v1r2 unit should simplify to second with any exponent
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isSecond();
+    }
   }
 
   delete ud;
@@ -460,18 +487,35 @@ UnitDefinition::isVariantOfTime () const
  * scale or multiplier values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfVolume () const
+UnitDefinition::isVariantOfVolume (bool relaxed) const
 {
   bool result = false;
 
   UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
   UnitDefinition::simplify(ud);
 
-  if (ud->getNumUnits() == 1)
+  if (!relaxed)
   {
-    const Unit* u = ud->getUnit(0);
-    result        = (u->isLitre() && u->getExponent() == 1) ||
-                    (u->isMetre() && u->getExponent() == 3);
+    // should be litre^1 or metre^3
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = (u->isLitre() && u->getExponent() == 1) ||
+                      (u->isMetre() && u->getExponent() == 3);
+    }
+  }
+  else
+  {
+    // should be any combination of litre/metre
+    unsigned int i = 0;
+    result = true;
+    while (result && i < ud->getNumUnits())
+    {
+      const Unit* u = ud->getUnit(i);
+      result = (  u->isLitre() || u->isMetre());
+      
+      i++;
+    }
   }
 
   delete ud;
@@ -485,17 +529,31 @@ UnitDefinition::isVariantOfVolume () const
  * or multiplier values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfDimensionless () const
+UnitDefinition::isVariantOfDimensionless (bool relaxed) const
 {
   bool result = false;
 
+  UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
+  UnitDefinition::simplify(ud);
 
-  if (getNumUnits() == 1)
+  if (!relaxed)
   {
-    const Unit* u = getUnit(0);
-    result        = u->isDimensionless();
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isDimensionless();
+    }
+  }
+  else
+  {
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = u->isDimensionless();
+    }
   }
 
+  delete ud;
   return result;
 }
 
@@ -506,18 +564,38 @@ UnitDefinition::isVariantOfDimensionless () const
  * values, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfMass () const
+UnitDefinition::isVariantOfMass (bool relaxed) const
 {
   bool result = false;
 
+  UnitDefinition *ud = static_cast<UnitDefinition*>(this->clone());
+  UnitDefinition::simplify(ud);
 
-  if (getNumUnits() == 1)
+  if (!relaxed)
   {
-    const Unit* u = getUnit(0);
-    result        = ((u->isGram() || u->isKilogram())
-                && u->getExponent() == 1);
+    // should be gram/kilogram ^ 1
+    if (ud->getNumUnits() == 1)
+    {
+      const Unit* u = ud->getUnit(0);
+      result        = ((u->isGram() || u->isKilogram())
+                  && u->getExponent() == 1);
+    }
+  }
+  else
+  {
+    // should be any combination of kilogram/gram
+    unsigned int i = 0;
+    result = true;
+    while (result && i < ud->getNumUnits())
+    {
+      const Unit* u = ud->getUnit(i);
+      result = (  u->isGram() || u->isKilogram());
+      
+      i++;
+    }
   }
 
+  delete ud;
   return result;
 }
 
@@ -527,7 +605,7 @@ UnitDefinition::isVariantOfMass () const
  * substance per time, false otherwise.
  */
 bool
-UnitDefinition::isVariantOfSubstancePerTime () const
+UnitDefinition::isVariantOfSubstancePerTime (bool relaxed) const
 {
   bool result = false;
 
@@ -542,8 +620,8 @@ UnitDefinition::isVariantOfSubstancePerTime () const
 
   UnitDefinition::simplify(ud);
 
-  result = ud->isVariantOfSubstance();
-
+  result = ud->isVariantOfSubstance(relaxed);
+  
   delete ud;
   delete u;
   return result;
@@ -561,11 +639,30 @@ UnitDefinition::addUnit (const Unit* u)
   {
     return returnValue;
   }
+  else if (u == NULL)
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+  else if (u->hasRequiredAttributes() == false)
+  {
+    return LIBSBML_INVALID_OBJECT;
+  }
+  else if (getLevel() != u->getLevel())
+  {
+    return LIBSBML_LEVEL_MISMATCH;
+  }
+  else if (getVersion() != u->getVersion())
+  {
+    return LIBSBML_VERSION_MISMATCH;
+  }
+  else if (matchesRequiredSBMLNamespacesForAddition(static_cast<const
+    SBase*>(u)) == false)
+  {
+    return LIBSBML_NAMESPACES_MISMATCH;
+  }
   else
   {
-    mUnits.append(u);
-
-    return LIBSBML_OPERATION_SUCCESS;
+    return mUnits.append(u);
   }
 }
 
@@ -660,7 +757,6 @@ UnitDefinition::removeUnit (unsigned int n)
 
 
 /** @cond doxygenLibsbmlInternal */
-
 /*
  * Sets the parent SBMLDocument of this SBML object.
  */
@@ -696,7 +792,6 @@ UnitDefinition::enablePackageInternal(const std::string& pkgURI,
 
   mUnits.enablePackageInternal(pkgURI,pkgPrefix,flag);
 }
-
 /** @endcond */
 
 
@@ -898,10 +993,14 @@ UnitDefinition::reorder(UnitDefinition *ud)
   int *initialIndexArray = NULL;
   initialIndexArray = new int[units->size()];
 
+  std::vector<unsigned int> used;
+
   for (n = 0; n < numUnits; n++)
   {
-    indexArray[n] = ((Unit *)units->get(n))->getKind();
-    initialIndexArray[n] = ((Unit *)units->get(n))->getKind();
+    unit = (Unit *)(units->get(n));
+    int value = (int)(unit->getKind());
+    indexArray[n] = value;
+    initialIndexArray[n] = value;
   }
 
   qsort(indexArray, numUnits, sizeof(int), compareKinds);
@@ -913,9 +1012,13 @@ UnitDefinition::reorder(UnitDefinition *ud)
     {
       if (indexArray[n] == initialIndexArray[p])
       {
-        unit = (Unit *) units->get(p);
-        units->append(unit);
-        break;
+        if (used.end() == std::find(used.begin(), used.end(), p)) 
+        {
+          unit = (Unit *) units->get(p);
+          units->append(unit);
+          used.push_back(p);
+          break;
+        }
       }
     }
   }
@@ -935,7 +1038,7 @@ UnitDefinition::reorder(UnitDefinition *ud)
  * Returns a UnitDefinition object which is the argument UnitDefinition
  * converted to the SI units.
  *
- * @param ud the UnitDefinition object to convert to SI
+ * @param ud the UnitDefinition object to convert to SI.
  *
  * @return a UnitDefinition object converted to SI units.
  */
@@ -1019,8 +1122,8 @@ extractMultiplier(UnitDefinition * ud)
  * Predicate returning @c true if 
  * UnitDefinition objects are identical (all units are identical).
  *
- * @param ud1 the first UnitDefinition object to compare
- * @param ud2 the second UnitDefinition object to compare
+ * @param ud1 the first UnitDefinition object to compare.
+ * @param ud2 the second UnitDefinition object to compare.
  *
  * @return @c true if all the units of ud1 are identical
  * to the units of ud2, @c false otherwise.
@@ -1126,8 +1229,8 @@ UnitDefinition::areIdentical(const UnitDefinition * ud1,
  * Predicate returning @c true if 
  * UnitDefinition objects are equivalent (all units are equivalent).
  *
- * @param ud1 the first UnitDefinition object to compare
- * @param ud2 the second UnitDefinition object to compare
+ * @param ud1 the first UnitDefinition object to compare.
+ * @param ud2 the second UnitDefinition object to compare.
  *
  * @return @c true if all the units of ud1 are equivalent
  * to the units of ud2, @c false otherwise.
@@ -1195,7 +1298,6 @@ UnitDefinition::areEquivalent(const UnitDefinition * ud1, const UnitDefinition *
 }
 
 /** @cond doxygenLibsbmlInternal */
-
 bool 
 UnitDefinition::areIdenticalSIUnits(const UnitDefinition * ud1, 
                                const UnitDefinition * ud2)
@@ -1292,7 +1394,6 @@ UnitDefinition::areIdenticalSIUnits(const UnitDefinition * ud1,
 
   return identical;
 }
-
 /** @endcond */
 
 /* 
@@ -1300,8 +1401,8 @@ UnitDefinition::areIdenticalSIUnits(const UnitDefinition * ud1,
  * which expresses the units of the two objects multiplied.
  *
  * @param ud1 the first UnitDefinition object into which the second is
- * combined
- * @param ud2 the second UnitDefinition object
+ * combined.
+ * @param ud2 the second UnitDefinition object.
  */
 UnitDefinition *
 UnitDefinition::combine(UnitDefinition *ud1, UnitDefinition *ud2)
@@ -1401,9 +1502,9 @@ UnitDefinition::divide(UnitDefinition *ud1, UnitDefinition *ud2)
  * @endcode
  * returns the string 'metre (exponent = 1) second (exponent = -2)'
  *
- * @param ud the UnitDefinition object
+ * @param ud the UnitDefinition object.
  *
- * @return a string expressing the units
+ * @return a string expressing the units.
  */
 std::string
 UnitDefinition::printUnits(const UnitDefinition * ud, bool compact)
@@ -1467,6 +1568,319 @@ UnitDefinition::printUnits(const UnitDefinition * ud, bool compact)
   }
   return unitDef;
 }
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::getAttribute(const std::string& attributeName,
+                             bool& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::getAttribute(const std::string& attributeName,
+                             int& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::getAttribute(const std::string& attributeName,
+                             double& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::getAttribute(const std::string& attributeName,
+                             unsigned int& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::getAttribute(const std::string& attributeName,
+                             std::string& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::getAttribute(const std::string& attributeName,
+                             const char* value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Predicate returning @c true if this UnitDefinition's attribute
+ * "attributeName" is set.
+ */
+bool
+UnitDefinition::isSetAttribute(const std::string& attributeName) const
+{
+  bool value = SBase::isSetAttribute(attributeName);
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::setAttribute(const std::string& attributeName, bool value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::setAttribute(const std::string& attributeName, int value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::setAttribute(const std::string& attributeName, double value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::setAttribute(const std::string& attributeName,
+                             unsigned int value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::setAttribute(const std::string& attributeName,
+                             const std::string& value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::setAttribute(const std::string& attributeName,
+                             const char* value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Unsets the value of the "attributeName" attribute of this UnitDefinition.
+ */
+int
+UnitDefinition::unsetAttribute(const std::string& attributeName)
+{
+  int value = SBase::unsetAttribute(attributeName);
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Creates and returns an new "elementName" object in this UnitDefinition.
+ */
+SBase*
+UnitDefinition::createChildObject(const std::string& elementName)
+{
+  SBase* obj = NULL;
+
+  if (elementName == "unit")
+  {
+    return createUnit();
+  }
+
+  return obj;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Returns the number of "elementName" in this UnitDefinition.
+ */
+unsigned int
+UnitDefinition::getNumObjects(const std::string& elementName)
+{
+  unsigned int n = 0;
+
+  if (elementName == "unit")
+  {
+    return getNumUnits();
+  }
+
+  return n;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Returns the nth object of "objectName" in this UnitDefinition.
+ */
+SBase*
+UnitDefinition::getObject(const std::string& elementName, unsigned int index)
+{
+  SBase* obj = NULL;
+
+  if (elementName == "unit")
+  {
+    return getUnit(index);
+  }
+
+  return obj;
+}
+
+/** @endcond */
+
+
 
 /* @cond doxygenLibsbmlInternal */
 /*
@@ -1494,6 +1908,7 @@ UnitDefinition::createObject (XMLInputStream& stream)
         logError(OneListOfUnitsPerUnitDef, getLevel(), getVersion());
       }
     }
+    mUnits.setExplicitlyListed();
     object = &mUnits;
   }
   
@@ -1528,7 +1943,7 @@ UnitDefinition::addExpectedAttributes(ExpectedAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 UnitDefinition::readAttributes (const XMLAttributes& attributes,
@@ -1559,7 +1974,7 @@ UnitDefinition::readAttributes (const XMLAttributes& attributes,
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 UnitDefinition::readL1Attributes (const XMLAttributes& attributes)
@@ -1587,7 +2002,7 @@ UnitDefinition::readL1Attributes (const XMLAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 UnitDefinition::readL2Attributes (const XMLAttributes& attributes)
@@ -1619,7 +2034,7 @@ UnitDefinition::readL2Attributes (const XMLAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 UnitDefinition::readL3Attributes (const XMLAttributes& attributes)
@@ -1631,23 +2046,43 @@ UnitDefinition::readL3Attributes (const XMLAttributes& attributes)
   //   id: SId     { use="required" }  (L2v1, L2v2)
   //
   bool assigned;
-  assigned = attributes.readInto("id", mId, getErrorLog(), false, getLine(), getColumn());
-  if (!assigned)
+  // for l3v2 sbase will read this as generically optional
+  // we want to log errors relating to the specific object
+  if (version == 1)
   {
-    logError(AllowedAttributesOnUnitDefinition, level, version, 
-             "The required attribute 'id' is missing.");
+    assigned = attributes.readInto("id", mId, getErrorLog(), false, getLine(), getColumn());
+    if (!assigned)
+    {
+      logError(AllowedAttributesOnUnitDefinition, level, version, 
+               "The required attribute 'id' is missing.");
+    }
+    if (assigned && mId.size() == 0)
+    {
+      logEmptyString("id", level, version, "<unitDefinition>");
+    }
+    if (!SyntaxChecker::isValidInternalSId(mId)) 
+      logError(InvalidIdSyntax, level, version, "The id '" + mId + "' does not conform to the syntax.");
   }
-  if (assigned && mId.size() == 0)
+  else
   {
-    logEmptyString("id", level, version, "<unitDefinition>");
+    // need to check that id was present
+    // it has already been read and checked for syntax/emptyness
+    if (attributes.hasAttribute("id") == false)
+    {
+      logError(AllowedAttributesOnUnitDefinition, level, version, 
+        "The required attribute 'id' is missing.");
+    }
   }
-  if (!SyntaxChecker::isValidInternalSId(mId)) 
-    logError(InvalidIdSyntax, level, version, "The id '" + mId + "' does not conform to the syntax.");
 
   //
   // name: string  { use="optional" }  (L2v1->)
   //
-  attributes.readInto("name", mName, getErrorLog(), false, getLine(), getColumn());
+  // for l3v2 sbase will read this
+  if (version == 1)
+  {
+    attributes.readInto("name", mName, getErrorLog(), false, 
+                                       getLine(), getColumn());
+  }
 }
 /** @endcond */
 
@@ -1655,7 +2090,7 @@ UnitDefinition::readL3Attributes (const XMLAttributes& attributes)
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write their XML attributes
- * to the XMLOutputStream.  Be sure to call your parents implementation
+ * to the XMLOutputStream.  Be sure to call your parent's implementation
  * of this method as well.
  */
 void
@@ -1664,18 +2099,30 @@ UnitDefinition::writeAttributes (XMLOutputStream& stream) const
   SBase::writeAttributes(stream);
 
   const unsigned int level = getLevel();
+  unsigned int version = getVersion();
 
-  //
-  // name: SName   { use="required" }  (L1v1, L1v2)
-  //   id: SId     { use="required" }  (L2v1, L2v2)
-  //
-  const string id = (level == 1) ? "name" : "id";
-  stream.writeAttribute(id, mId);
-
-  //
-  // name: string  { use="optional" }  (L2v1->)
-  //
-  if (level > 1) stream.writeAttribute("name", mName);
+  // for L3V2 and above SBase will write this out
+  if (level < 3 || (level == 3 && version == 1))
+  {
+    //
+    // name: SName   { use="required" }  (L1v1, L1v2)
+    //   id: SId     { use="required" }  (L2v1, L2v2)
+    //
+    const string id = (level == 1) ? "name" : "id";
+    stream.writeAttribute(id, mId);
+  }
+  
+  if (level > 1)
+  {
+    // for L3V2 and above SBase will write this out
+    if (level < 3 || (level == 3 && version == 1))
+    {
+      //
+      // name: string  { use="optional" }  (L2v1->)
+      //
+      stream.writeAttribute("name", mName);
+    }
+  }
 
   //
   // sboTerm: SBOTerm { use="optional" }  (L2v3->)
@@ -1693,14 +2140,28 @@ UnitDefinition::writeAttributes (XMLOutputStream& stream) const
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write out their contained
- * SBML objects as XML elements.  Be sure to call your parents
+ * SBML objects as XML elements.  Be sure to call your parent's
  * implementation of this method as well.
  */
 void
 UnitDefinition::writeElements (XMLOutputStream& stream) const
 {
   SBase::writeElements(stream);
-  if ( getNumUnits() > 0 ) mUnits.write(stream);
+
+  if (getLevel() == 3 && getVersion() > 1)
+  {
+    if (mUnits.hasOptionalElements() == true ||
+        mUnits.hasOptionalAttributes() == true ||
+        mUnits.isExplicitlyListed())
+    {
+      mUnits.write(stream);
+    }
+  }
+  else
+  {
+    // use original code
+    if ( getNumUnits() > 0 ) mUnits.write(stream);
+  }
 
   //
   // (EXTENSION)
@@ -1782,11 +2243,11 @@ ListOfUnitDefinitions::get(unsigned int n) const
  */
 struct IdEqUD : public unary_function<SBase*, bool>
 {
-  const string& id;
+  const string& mId;
 
-  IdEqUD (const string& id) : id(id) { }
+  IdEqUD (const string& id) : mId(id) { }
   bool operator() (SBase* sb) 
-       { return static_cast <UnitDefinition *> (sb)->getId() == id; }
+       { return static_cast <UnitDefinition *> (sb)->getId() == mId; }
 };
 
 
@@ -1805,16 +2266,9 @@ ListOfUnitDefinitions::get (const std::string& sid) const
 {
   vector<SBase*>::const_iterator result;
 
-  if (&(sid) == NULL)
-  {
-    return NULL;
-  }
-  else
-  {
-    result = find_if( mItems.begin(), mItems.end(), IdEqUD(sid) );
-    return (result == mItems.end()) ? NULL : 
-                     static_cast <UnitDefinition*> (*result);
-  }
+  result = find_if( mItems.begin(), mItems.end(), IdEqUD(sid) );
+  return (result == mItems.end()) ? NULL : 
+                   static_cast <UnitDefinition*> (*result);
 }
 
 
@@ -1847,15 +2301,12 @@ ListOfUnitDefinitions::remove (const std::string& sid)
   SBase* item = NULL;
   ListItemIter result;
 
-  if (&(sid) != NULL)
-  {
-    result = find_if( mItems.begin(), mItems.end(), IdEqUD(sid) );
+  result = find_if( mItems.begin(), mItems.end(), IdEqUD(sid) );
 
-    if (result != mItems.end())
-    {
-      item = *result;
-      mItems.erase(result);
-    }
+  if (result != mItems.end())
+  {
+    item = *result;
+    mItems.erase(result);
   }
 
   return static_cast <UnitDefinition*> (item);
@@ -1914,8 +2365,6 @@ ListOfUnitDefinitions::createObject (XMLInputStream& stream)
 
 #endif /* __cplusplus */
 /** @cond doxygenIgnored */
-
-
 LIBSBML_EXTERN
 UnitDefinition_t *
 UnitDefinition_create (unsigned int level, unsigned int version)
@@ -2246,8 +2695,6 @@ ListOfUnitDefinitions_removeById (ListOf_t *lo, const char *sid)
   else
     return NULL;
 }
-
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_END
